import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sun.jna.Library;
import com.sun.jna.Native;

public class AnapicoLANDemo {
	private static final int VI_SUCCESS = (int) Long.parseLong("00000000", 16);
	private static final int VI_SUCCESS_MAX_CNT = (int) Long.parseLong("3FFF0006", 16);
	private static final int VI_ERROR_SYSTEM_ERROR = (int) Long.parseLong("BFFF0000", 16);
	private static final int VI_ERROR_TMO = (int) Long.parseLong("BFFF0015", 16);
	private static final int VI_ERROR_CONN_LOST = (int) Long.parseLong("BFFF00A6", 16);
	private static final int VI_ERROR_FILE_ACCESS = (int) Long.parseLong("BFFF00A1", 16);

	public interface LANInterface extends Library {
		public int ap_sock_find(byte[] devices, int tmoMs, int count, int[] retCount);

		public int ap_sock_open(byte[] adress, int port, int tmoMs, int[] sock_id);

		public int ap_sock_close(int sock_id);

		public int ap_sock_puts(int sock_id, byte[] buf, int count, int[] retCount);

		public int ap_sock_gets(int sock_id, byte[] buf, int count, int[] retCount);

		public int ap_sock_read(int sock_id, byte[] i16buf, int size);

		public int ap_sock_clear(int sock_id);

		public int ap_sock_settmo(int sock_id, int tmoMs);
	}

	public String retToString(int ret) {
		if (ret == VI_SUCCESS) return "VI_SUCCESS";
		else if (ret == VI_SUCCESS_MAX_CNT) return "VI_SUCCESS_MAX_CNT";
		else if (ret == VI_ERROR_CONN_LOST) return "VI_ERROR_CONN_LOST";
		else if (ret == VI_ERROR_FILE_ACCESS) return "VI_ERROR_FILE_ACCESS";
		else if (ret == VI_ERROR_SYSTEM_ERROR) return "VI_ERROR_SYSTEM_ERROR";
		else if (ret == VI_ERROR_TMO) return "VI_ERROR_TMO";
		return "UNKNOWN";
	}

	private int[] retCount = new int[1];

	public AnapicoLANDemo() {
		LANInterface lib = null;
		try {
			lib = (LANInterface) Native.loadLibrary("lan/ap_lan_lib", LANInterface.class);
		} catch (UnsatisfiedLinkError e) {
			System.out.println("Unable to load ap_lan_lib.dll.");
			return;
		}
		System.out.println("ap_lan_lib.dll loaded.\n");

		System.out.println("Search for devices:");
		byte[] devices = new byte[1024];
		int res = lib.ap_sock_find(devices, 1000, 1024, retCount);
		System.out.printf("ap_sock_find(byte[1024], 1000, 1024, int[]) > %s\n", retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;
		System.out.printf(" > %d bytes: '%s'\n", retCount[0], new String(devices).trim());

		// ask user what device to take
		String[] devs = new String(devices).trim().split(";");
		for (int i = 0; i < devs.length; i++)
			if (devs[i].contains("APPH")) System.out.printf("  %d: %s\n", i, devs[i]);
		System.out.println("Choose device: ");
		Scanner scan = new Scanner(System.in);
		String ipAddr = extractIP(devs[scan.nextInt()]);
		if (ipAddr == null) return;
		System.out.println();

		System.out.println("Connect to first device:");
		int[] sockID = new int[1];
		ipAddr += '\000'; // Java doesn't use null terminated strings, therefore we have to add it manually 
		res = lib.ap_sock_open(ipAddr.getBytes(), 18, 2000, sockID);
		System.out.printf("ap_sock_open('%s') -> sockID=%d > %s\n\n", ipAddr.trim(), sockID[0], retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;
		if (sockID[0] <= 0) return; // not connected

		System.out.println("Send a command:");
		String cmd = "FREQ 100000000\n";
		res = lib.ap_sock_puts(sockID[0], cmd.getBytes(), cmd.getBytes().length, retCount);
		System.out.printf("ap_sock_puts('%s') > %s\n\n", cmd.trim(), retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;

		System.out.println("Send a Query and retrieve the answer:");
		cmd = "*IDN?\n";
		res = lib.ap_sock_puts(sockID[0], cmd.getBytes(), cmd.getBytes().length, retCount);
		System.out.printf("ap_usb_puts('%s') > %s\n", cmd.trim(), retToString(res));
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;
		byte[] buf = new byte[256];
		res = lib.ap_sock_gets(sockID[0], buf, buf.length, retCount);
		System.out.printf("ap_usb_gets(%d) > %s\n", buf.length, retToString(res));
		String result = new String(buf).substring(0, retCount[0]).trim();
		System.out.printf(" > %d bytes: '%s'\n\n", retCount[0], result);
		if (res != VI_SUCCESS && res != VI_SUCCESS_MAX_CNT) return;

		System.out.println("Disconnect:");
		res = lib.ap_sock_close(sockID[0]);
		System.out.printf("ap_sock_close() > %s\n\n", retToString(res));
	}

	private String extractIP(String string) {
		String[] splitString = string.split(":");
		if (splitString.length >= 2 && isIP(splitString[2])) return splitString[2];
		return null;
	}

	private boolean isIP(String string) {
		Matcher deviceMatcher = Pattern.compile("^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$").matcher(string);
//		Matcher deviceMatcher = Pattern.compile("^\\b(?:\\d{1,3}\\.){3}\\d{1,3}\\b$").matcher(string); // simpler but less restrictive regex for IPv4
		return deviceMatcher.matches();
	}

	public static void main(String[] args) {
		new AnapicoLANDemo();
	}
}
